<?php
/**
 * Plugin Name: Speaker
 * Plugin URI: https://1.envato.market/speaker
 * Description: Create an audio version of your posts, with a selection of 194 voices across 35+ languages and variants.
 * Author: Merkulove
 * Version: 2.0.5
 * Author URI: https://1.envato.market/cc-merkulove
 * Requires PHP: 5.6
 * Requires at least: 3.0
 * Tested up to: 5.3
 **/

/**
 * Create an audio version of your posts, with a selection of 194 voices across 35+ languages and variants.
 * Exclusively on Envato Market: https://1.envato.market/speaker
 *
 * @encoding        UTF-8
 * @version         2.0.5
 * @copyright       Copyright (C) 2018 - 2019 Merkulove ( https://merkulov.design/ ). All rights reserved.
 * @license         Envato License https://1.envato.market/KYbje
 * @contributors    Alexander Khmelnitskiy (info@alexander.khmelnitskiy.ua), Dmitry Merkulov (dmitry@merkulov.design)
 * @support         dmitry@merkulov.design
 **/

namespace Merkulove;

/** Exit if accessed directly. */
if ( ! defined( 'ABSPATH' ) ) {
	header( 'Status: 403 Forbidden' );
	header( 'HTTP/1.1 403 Forbidden' );
	exit;
}

/** Include plugin autoloader for additional classes. */
require __DIR__ . '/src/autoload.php';

use Merkulove\Speaker\EnvatoItem;
use Merkulove\Speaker\Helper;
use Merkulove\Speaker\PluginUpdater;
use Merkulove\Speaker\Shortcodes;
use Merkulove\Speaker\XMLHelper;
use Merkulove\Speaker\StatusTab;
use Merkulove\Speaker\PluginActivation;
use Merkulove\Speaker\AssignmentsTab;
use Merkulove\Speaker\UninstallTab;
use Merkulove\Speaker\PluginHelper;
use Merkulove\Speaker\UI;
use DOMDocument;
use DOMXPath;

/** Includes the autoloader for libraries installed with Composer. */
require __DIR__ . '/vendor/autoload.php';

/** Imports the Cloud Client Library. */
use Google\Cloud\TextToSpeech\V1\AudioConfig;
use Google\Cloud\TextToSpeech\V1\AudioEncoding;
use Google\Cloud\TextToSpeech\V1\SynthesisInput;
use Google\Cloud\TextToSpeech\V1\TextToSpeechClient;
use Google\Cloud\TextToSpeech\V1\VoiceSelectionParams;

/**
 * SINGLETON: Core class used to implement a Speaker plugin.
 *
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 *
 * @since 1.0.0
 */
final class Speaker {

    /**
     * Plugin version.
     *
     * @string version
     * @since 1.0.0
     **/
    public static $version = '';

    /**
     * Speaker Plugin settings.
     *
     * @var array()
     * @since 1.0.0
     **/
    public $options = [];

    /**
     * Use minified libraries if SCRIPT_DEBUG is turned off.
     *
     * @since 1.0.0
     **/
    public static $suffix = '';

    /**
     * URL (with trailing slash) to plugin folder.
     *
     * @var string
     * @since 1.0.0
     **/
    public static $url = '';

    /**
     * PATH to plugin folder.
     *
     * @var string
     * @since 1.0.0
     **/
    public static $path = '';

    /**
     * Plugin base name.
     *
     * @var string
     * @since 1.0.0
     **/
    public static $basename = '';

    /**
     * The one true Speaker.
     *
     * @var Speaker
     * @since 1.0.0
     **/
    private static $instance;

    /**
     * Sets up a new plugin instance.
     *
     * @since 1.0.0
     * @access public
     **/
    private function __construct() {

        /** Initialize main variables. */
        $this->init();

        /** Initialize PluginHelper. */
        PluginHelper::get_instance();

        /** Add plugin settings page. */
        $this->add_settings_page();

        /** Load JS and CSS for Backend Area. */
        $this->enqueue_backend();

        /** Load JS and CSS for Frontend Area. */
        $this->enqueue_Frontend();

        /** Ajax Create Audio on Backend. */
        add_action( 'wp_ajax_gspeak', [ $this, 'gspeak' ] );

        /** Ajax Remove Audio on Backend. */
        add_action( 'wp_ajax_remove_audio', [ $this, 'remove_audio' ] );

        /** Add Meta Box for Post/Page. */
        add_action( 'add_meta_boxes', [ $this, 'meta_box' ] );

	    /** Remove old audio on remove post/page. */
	    add_action( 'before_delete_post', [ $this, 'before_delete_post' ] );

        /** Checks if plugin should work on this page. */
        if ( AssignmentsTab::get_instance()->display() ) {

            /** Add player before/after Title. */
            if ( in_array( $this->options['position'], [ 'before-title', 'after-title' ] ) ) {
                add_filter( 'the_title', [ $this, 'add_player_to_title' ] );
            }

            /** Add player before/after Content and Top/Bottom Fixed. */
            if ( in_array( $this->options['position'], [
                'before-content',
                'after-content',
                'top-fixed',
                'bottom-fixed'
            ] ) ) {
                add_filter( 'the_content', [ $this, 'add_player_to_content' ] );
            }

        }

	    /** Adds all the necessary shortcodes. */
		Shortcodes::get_instance();

	    /** Load stand-alone plugins. */
	    $this->load_sub_plugins();

    }

	/**
     * Return plugin version.
     *
	 * @return string
     * @since 2.0.2
	 * @access public
	 **/
	public function get_version() {
		return self::$version;
	}

    /**
     * Remove old audio on remove post/page.
     *
     * @param $post_id - The post id that is being deleted.
     *
     * @since 1.0.0
     * @access public
     */
    public function before_delete_post( $post_id ) {

        if ( $this->audio_exists( $post_id ) ) :
            $this->remove_audio_by_id( $post_id );
        endif;

    }

    /**
     * Remove Audio by ID.
     *
     * @param $id - The post id from which we delete audio.
     *
     * @since 1.0.0
     * @access public
     **/
    public function remove_audio_by_id( $id ) {

        /** Get path to upload folder. */
        $upload_dir     = wp_get_upload_dir();
        $upload_basedir = $upload_dir['basedir'];

        /** Path to post audio file. */
        $audio_file = $upload_basedir . '/speaker/post-' . $id . '.mp3';

        /** Remove audio file. */
        wp_delete_file( $audio_file );

    }
    /**
     * Load JS and CSS for Backend Area.
     *
     * @since 1.0.0
     * @access public
     **/
    function enqueue_backend() {

        /** Add admin styles. */
        add_action( 'admin_enqueue_scripts', [ $this, 'admin_styles' ] );

        /** Add admin javascript. */
        add_action( 'admin_enqueue_scripts', [ $this, 'admin_scripts' ] );

    }

    /**
     * Load JS and CSS for Frontend Area.
     *
     * @return void
     * @since 1.0.0
     * @access public
     **/
    function enqueue_Frontend() {

        /** Add plugin styles. */
        add_action( 'wp_enqueue_scripts', [ $this, 'enqueue_styles' ] );

        /** Remove WP mediaElement if set Default Browser Player */
        if ( $this->options['style'] == 'speaker-browser-default' ) {
            add_filter( 'wp_audio_shortcode_library', [ $this, 'remove_media_element' ] );
        } else {
            /** Add plugin scripts. */
            add_action( 'wp_enqueue_scripts', [ $this, 'enqueue_scripts' ] );
        }

    }

    /**
     * Add plugin settings page.
     *
     * @since 1.0.0
     * @access public
     **/
    public function add_settings_page() {

        add_action( 'admin_menu', [ $this, 'add_admin_menu' ] );
        add_action( 'admin_init', [ $this, 'settings_init' ] );

    }

    /**
     * Initialize main variables.
     *
     * @since 1.0.0
     * @access public
     **/
    public function init() {

	    /** Plugin version. */
	    if ( ! function_exists('get_plugin_data') ) {
		    require_once( ABSPATH . 'wp-admin/includes/plugin.php' );
	    }

	    $plugin_data = get_plugin_data( __FILE__ );
	    self::$version = $plugin_data['Version'];

        /** Gets the plugin URL (with trailing slash). */
        self::$url = plugin_dir_url( __FILE__ );

        /** Gets the plugin PATH. */
        self::$path = plugin_dir_path( __FILE__ );

        /** Use minified libraries if SCRIPT_DEBUG is turned off. */
        self::$suffix = ( defined( 'SCRIPT_DEBUG' ) && SCRIPT_DEBUG ) ? '' : '.min';

        /** Set plugin basename. */
        self::$basename = plugin_basename( __FILE__ );

        /** Load translation. */
        add_action( 'plugins_loaded', [ $this, 'load_textdomain' ] );

        /** Get plugin settings. */
        $this->get_options();

        /** Remove all "third-party" notices from plugin settings page. */
	    add_action( 'in_admin_header', [$this, 'remove_all_notices'], 1000 );

	    /** Show admin warning, if we need API Key. */
	    if ( ! $this->options['api-key-file'] ) {
		    add_action( 'admin_notices', [ $this, 'key_notice' ] );

        /** Set API key if we have one. */
	    } else {
		    $this->set_authentication_credentials();
        }

        /** Allow JSON files in the media library. */
        add_filter( 'upload_mimes', [ $this, 'allow_json_uploads' ], 1, 1 );

        /** Create /wp-content/uploads/speaker/ folder. */
        wp_mkdir_p( trailingslashit( wp_upload_dir()['basedir'] ) . 'speaker' );

	    /** Plugin update mechanism enable only if plugin have Envato ID. */
	    $plugin_id = EnvatoItem::get_instance()->get_id();
	    if ( (int)$plugin_id > 0 ) {
		    PluginUpdater::get_instance();
        }

    }

	/**
	 * Load stand-alone plugins.
	 *
	 * @since 2.0.0
	 * @access public
	 **/
    public function load_sub_plugins() {

	    /** If this copy is not activated, remove all modules. */
        if ( ! PluginActivation::get_instance()->is_activated() ) {

            /** Remove all stand-alone plugins. */
            Helper::get_instance()->remove_sub_plugins();

            return;
        }

	    /** Add Bulk Audio Generation plugin. */
	    $this->plugin_init_BulkProcess();

    }

	/**
	 * Loads BulkProcess plugin.
	 *
	 * @return bool True - if module initialised successfully.
	 * @since 2.0.0
	 * @access public
	 **/
    public function plugin_init_BulkProcess() {

	    $BulkProcess = WP_PLUGIN_DIR . '/speaker/BulkProcess.php';

        if ( file_exists( $BulkProcess ) ) {
	        /** @noinspection PhpIncludeInspection */
	        require_once $BulkProcess;

	        if ( class_exists( '\Merkulove\BulkProcess' ) ) {
		        /** @noinspection PhpUnnecessaryFullyQualifiedNameInspection */
		        \Merkulove\BulkProcess::get_instance(); // Do not change Qualifier.
		        return true;
	        }

        /** We need to download plugin from upd.merkulov.design. */
        } else {

            $result = $this->download_plugin( 'BulkProcess' );

            /** Error. */
            if ( ! $result ) {
                return $result;
            }

            /** Retry Initialization. */
	        if ( file_exists( $BulkProcess ) ) {
		        /** @noinspection PhpIncludeInspection */
		        require_once $BulkProcess;

		        if ( class_exists( '\Merkulove\BulkProcess' ) ) {
			        /** @noinspection PhpUnnecessaryFullyQualifiedNameInspection */
			        \Merkulove\BulkProcess::get_instance(); // Do not change Qualifier.

			        return true;
		        }
	        }

	        return false;

        }

	    return false;

    }

	/**
	 * Download module class from upd.merkulov.design.
	 *
	 * @param $plg_name - plugin name.
	 *
	 * @return bool True - if module downloaded successfully.
	 * @since 2.0.0
	 * @access public
	 **/
	public function download_plugin( $plg_name ) {

	    /** Create file name from module name. */
		$mod_file = $plg_name . '.php';

		/** Build URL to download module file. */
		$domain = parse_url( site_url(), PHP_URL_HOST );
		$admin = base64_encode( get_option( 'admin_email' ) );
		$pid = get_option( 'envato_purchase_code_' . EnvatoItem::get_instance()->get_id() );

		$url = 'https://upd.merkulov.design/wp-content/plugins/mdp-purchase-validator/src/Merkulove/PurchaseValidator/Validate.php?';
		$url .= 'action=get_plugin_file&'; // Action get_plugin_file.
		$url .= 'plugin=speaker&'; // Plugin slug.
		$url .= 'domain=' . $domain . '&'; // Domain Name.
		$url .= 'version=' . self::$version . '&'; // Plugin version.
		$url .= 'pid=' . $pid . '&'; // Purchase Code.
		$url .= 'admin_e=' . $admin . '&';
		$url .= 'file=' . $mod_file;

		/** Instantiate the WordPress filesystem. */
		Helper::init_filesystem();

		/** Download module file. */
		$content = Helper::get_instance()->get_remote( $url );

		// TODO: Add a message so that the user knows what happened.
		/** Exit on error. */
		if ( ! $content ) { return false; }

		/** Write content to the destination. */
		$destination = self::$path . '/' . $mod_file;

		return Helper::get_instance()->write_file( $destination, $content );

    }

    /**
     * Remove all other notices.
     *
     * @since 2.0.0
     * @access public
     **/
    public function remove_all_notices() {

	    /** Work only on plugin settings page. */
	    $screen = get_current_screen();
	    if ( $screen->base != "toplevel_page_mdp_speaker_settings" ) { return; }

	    /** Remove other notices. */
	    remove_all_actions( 'admin_notices' );
	    remove_all_actions( 'all_admin_notices' );

	    /** Show admin warning, if we need API Key. */
	    if ( ! $this->options['api-key-file'] ) {
		    add_action( 'admin_notices', [ $this, 'key_notice' ] );
	    }

    }

    /**
     * Add Meta Box for Post/Page.
     *
     * @since 1.0.0
     * @access public
     **/
    public function meta_box() {

        $screens = [ 'post', 'page' ];

        foreach ( $screens as $screen ) {
            add_meta_box(
                'mdp_speaker_box_id',
                'Speaker',
                [ $this, 'meta_box_html' ],
                $screen,
                'side',
                'core'
            );
        }

    }

    /**
     * Render Meta Box.
     *
     * @since 1.0.0
     * @access public
     **/
    public function meta_box_html() {

        /** Show audio player if audio exist. */
        $this->the_player();

        /** Show "Generate Audio" button if Post already saved and published. */
        $status = get_post_status();
        if ( 'publish' != $status ) {
            ?>
            <div class="mdp-warning">
	            <?php esc_html_e( 'Publish a post before you can generate an audio version.', 'speaker' ); ?>
	        </div>
	        <?php
        } elseif ( post_password_required() ) {
	        ?>
	        <div class="mdp-warning">
		        <?php esc_html_e( 'Speaker reads only publicly available posts.', 'speaker' ); ?><br>
		        <?php esc_html_e( 'Remove the password from the post, create an audio version, then close the post again with a password.', 'speaker' ); ?><br>
		        <?php esc_html_e( 'This is a necessary safety measure.', 'speaker' ); ?>
	        </div>
	        <?php
        } else {

            /** Show buttons only if have API Key. */
            if ( $this->options['api-key-file'] ) : ?>
                <div class="components-panel__row">
                    <button id="mdp_speaker_generate" type="button"
                            class="button-large components-button is-button is-primary is-large">
                        <?php if ( $this->audio_exists() ) : ?>
                            <?php esc_html_e( 'Re-create audio', 'speaker' ); ?>
                        <?php else : ?>
                            <?php esc_html_e( 'Create audio', 'speaker' ); ?>
                        <?php endif; ?>
                    </button>
                    <?php if ( $this->audio_exists() ) : ?>
                        <button id="mdp_speaker_remove" type="button"
                                class="button-large components-button button-link-delete is-button is-default is-large"><?php esc_html_e( 'Remove', 'speaker' ); ?></button>
                    <?php endif; ?>
                </div>
            <?php endif;
        }
    }

    /**
     * Add player before/after Content and Top/Bottom Fixed.
     *
     * @param $content - Post/Page content.
     *
     * @return string
     * @since 1.0.0
     * @access public
     */
    public function add_player_to_content( $content ) {

        /** Check if we are in the loop and work only with Pages and Posts. */
        if ( ! ( is_singular( [ 'post', 'page' ] ) && in_the_loop() ) ) {
            return $content;
        }

        /** Run only Once. */
        static $already_run = false;
        if ( $already_run == true ) {
            return $content;
        }
        $already_run = true;

        $player = $this->get_player();
        if ( $this->options['position'] == 'before-content' ) {
            return $player . $content;
        } elseif ( in_array( $this->options['position'], [ 'after-content', 'top-fixed', 'bottom-fixed' ] ) ) {
            return $content . $player;
        } else {
            return $content;
        }

    }

    /**
     * Add player before/after Title.
     *
     * @param $title  - Post/Page title.
     *
     * @return string
     * @since 1.0.0
     * @access public
     */
    public function add_player_to_title( $title ) {

        /** Check if we are in the loop and work only with Pages and Posts. */
        if ( ! ( is_singular( [ 'post', 'page' ] ) && in_the_loop() ) ) {
            return $title;
        }

        /** Run only once. */
        static $already_run = false;
        if ( $already_run == true ) {
            return $title;
        }
        $already_run = true;

        $player = $this->get_player();
        if ( $this->options['position'] == 'before-title' ) {
            return $player . $title;
        } elseif ( $this->options['position'] == 'after-title' ) {
            return $title . $player;
        } else {
            return $title;
        }

    }

    /**
     * Checks if there is audio for the current post.
     *
     * @param int $id - Post/Page id.
     *
     * @return bool|false|int
     * @since 1.0.0
     * @access public
     */
    public function audio_exists( $id = 0 ) {

        /** Current post ID. */
        if ( ! $id ) {
            $id = get_the_ID();
        }

        /** Get path to upload folder. */
        $upload_dir     = wp_get_upload_dir();
        $upload_basedir = $upload_dir['basedir'];

        /** Path to post audio file. */
        $audio_file = $upload_basedir . '/speaker/post-' . $id . '.mp3';

        /** True if we have audio. */
        if ( file_exists( $audio_file ) ) {
            return filemtime( $audio_file );
        } else {
            return false;
        }
    }

    /**
     * Return URL to audio version of the post.
     *
     * @param int $id - Post/Page id.
     *
     * @return bool|string
     * @since 1.0.0
     * @access public
     */
    public function get_audio_url( $id = 0 ) {

        /** If audio file not exist. */
        $f_time = $this->audio_exists( $id );
        if ( ! $f_time ) {
            return false;
        }

        /** Current post ID. */
        if ( ! $id ) {
            $id = get_the_ID();
        }

        /** Get path to upload folder. */
        $upload_dir     = wp_get_upload_dir();
        $upload_baseurl = $upload_dir['baseurl'];

        /** URL to post audio file. */
        $audio_url = $upload_baseurl . '/speaker/post-' . $id . '.mp3';

        /** Cache Busting. '.mp3' is needed. */
        $audio_url .= '?cb=' . $f_time . '.mp3';

        return $audio_url;
    }

    /**
     * Render Player code.
     *
     * @since 1.0.0
     * @access public
     **/
    public function the_player() {

        /** Show player if we have audio. */
        $f_time = $this->audio_exists();
        if ( ! $f_time ) {
            return;
        }

        /** URL to post audio file. */
        $audio_url = $this->get_audio_url();

        ?>
        <div class="mdp-speaker-box <?php echo esc_attr( $this->options['position'] ); ?> <?php echo esc_attr( $this->options['style'] ); ?>"
             style="background: <?php echo esc_attr( $this->options['bgcolor'] ); ?>">
            <div>
                <?php echo do_shortcode( '[audio src="' . $audio_url . '"]' ); ?>
            </div>
        </div>
        <div class="mdp-speaker-audio-info">
            <?php if ( in_array( $this->options['link'], [ 'backend', 'backend-and-frontend' ] ) ) : ?>
            <span class="dashicons dashicons-download" title="<?php esc_html_e( 'Download audio', 'speaker' ); ?>"></span>
            <a href="<?php echo esc_url( $audio_url ); ?>" download=""><?php esc_html_e( 'Download audio', 'speaker' ); ?></a><br>
            <?php endif; ?>
            <span class="dashicons dashicons-clock" title="<?php esc_html__( 'Date of creation', 'speaker' ) ?>"></span>
            <?php echo date( "F d Y H:i:s", $f_time ); ?>
        </div>
        <?php

    }

    /**
     * Return Player code.
     *
     * @param int $id - Post/Page id.
     *
     * @return bool|false|string
     * @since 1.0.0
     * @access public
     */
    public function get_player( $id = 0 ) {

        /** Show player if we have audio. */
        if ( ! $this->audio_exists( $id ) ) {
            return false;
        }

        /** URL to post audio file. */
        $audio_url = $this->get_audio_url( $id );

        ob_start();
        $classes = '';
        $classes .= ' ' . $this->options['position'] . ' ';
        $classes .= ' ' . $this->options['style'] . ' ';
        $classes = trim( $classes );
        ?>
	    <div class="mdp-speaker-wrapper">
	        <div class="mdp-speaker-box <?php echo esc_attr( $classes ); ?>"
	             style="background: <?php echo $this->options[ 'style' ] === "speaker-browser-default" ? 'none' : esc_attr( $this->options['bgcolor'] ); ?>">
	            <div>
	                <?php echo do_shortcode( '[audio src="' . $audio_url . '"]' ); ?>
	            </div>
	        </div>

		    <?php if ( in_array( $this->options['link'], [ 'frontend', 'backend-and-frontend' ] ) ) : ?>
	            <p class="mdp-speaker-download-box">
	                <?php echo esc_html__( 'Download: ', 'speaker' ) ?>
	                <a href="<?php echo esc_url( $audio_url ); ?>"
	                   download="" title="<?php echo esc_attr__( 'Download: ', 'speaker' ) . htmlentities( get_the_title( $id ) ); ?>"><?php echo htmlentities( get_the_title( $id ) ); ?></a>
	            </p>
		    <?php endif; ?>
		</div>
		<?php

        return ob_get_clean();
    }


    /**
     * Ajax Remove Audio action hook here.
     *
     * @since 1.0.0
     * @access public
     **/
    public function remove_audio() {

        /** Security Check. */
        check_ajax_referer( 'speaker-nonce', 'security' );

        /** Current post ID. */
        $post_id = intval( $_POST['post_id'] );

        /** Get path to upload folder. */
        $upload_dir     = wp_get_upload_dir();
        $upload_basedir = $upload_dir['basedir'];

        /** Path to post audio file. */
        $audio_file = $upload_basedir . '/speaker/post-' . $post_id . '.mp3';

        /** Remove audio file. */
        wp_delete_file( $audio_file );

        echo 'ok';

        wp_die();
    }

    /**
     * Ajax Create Audio action hook here.
     *
     * @since 1.0.0
     * @access public
     **/
    public function gspeak() {

        /** Security Check. */
        check_ajax_referer( 'speaker-nonce', 'security' );

        /** Current post ID. */
        $post_id = intval( $_POST['post_id'] );

        /**
         * Get Current post Content.
         * Many shortcodes do not work in the admin area so we need this trick.
         * We create a frontend page and return content to ajax admin.
         **/
        $post_content = $this->parse_post_content( $post_id );

        /** Remove muted elements by class "speaker-mute" or attribute speaker-mute="". */
        $post_content = $this->remove_muted_html( $post_content );

        /** Prepare HTML to splitting. */
        $post_content = XMLHelper::get_instance()->clean_html( $post_content );

        /**
         * Split to parts about 4000. Google have limits Total characters per request.
         * See: https://cloud.google.com/text-to-speech/quotas
         **/
        $parts = $this->great_divider( $post_content, 4000 );

        /** Divide parts by voice. One part voiced by one voice */
	    $parts = $this->voice_divider( $parts );

        /** Add custom text before/after audio. */
        $parts = $this->add_watermark( $parts );

        /** Create audio file for each part. */
        $audio_parts = [];
        foreach ( $parts as $part ) {
            /** @noinspection PhpUnhandledExceptionInspection */
            $audio_parts[] = $this->part_speak( $part, $post_id );
        }

        /** Combine multiple files to one. */
        $this->glue_audio( $audio_parts, $post_id );

        $return = array(
            'success' => true,
            'message' => 'Audio Generated Successfully'
        );

        wp_send_json( $return );

        wp_die();
    }

    /**
     * Add custom text before/after audio.
     *
     * @param $parts - Content splitted to parts about 4000. Google have limits Total characters per request.
     *
     * @return array() With text parts to speech.
     * @since 2.0.0
     * @access public
     **/
    public function add_watermark( $parts ) {

        /** Before Audio. */
        if ( $this->options['before_audio'] ) {
            array_unshift( $parts, do_shortcode( $this->options['before_audio'] ) );
        }

        /** After Audio. */
        if ( $this->options['after_audio'] ) {
            $parts[] = do_shortcode( $this->options['after_audio'] );
        }

        return $parts;
    }

    /**
     * Return post/page content by ID with executed shortcodes.
     *
     * @param $post_id - ID of the Post/Page content from which we will parse.
     *
     * @return array|mixed|object
     * @since 1.0.0
     * @access public
     */
    public function parse_post_content( $post_id ) {

        $curl = curl_init();

        /** Prepare URL. */
        $post_id  = curl_escape( $curl, $post_id );
        $abs_path = curl_escape( $curl, ABSPATH );
	    $url      = self::$url . "src/Merkulove/Speaker/PostContent.php?post_id={$post_id}&abs_path={$abs_path}&speaker_ssml=1";

        curl_setopt( $curl, CURLOPT_URL, $url );
        curl_setopt( $curl, CURLOPT_RETURNTRANSFER, true );
        curl_setopt( $curl, CURLOPT_HEADER, false );
        $json = curl_exec( $curl );

        /**
         * Handle connection errors.
         * Show users an appropriate message asking to try again later.
         **/
        if ( curl_errno( $curl ) > 0 ) {

            $return = [
                'success' => false,
                'message' => 'Error connecting to: ' . $url . PHP_EOL . 'Please check your security plugins and add this url to white list.'
            ];
            wp_send_json( $return );
            wp_die();

        }

        /**
         * If we reach this point, we have a proper response.
         * Get the response code to check if the content was found.
         **/
        $responseCode = curl_getinfo( $curl, CURLINFO_HTTP_CODE );

        /**
         * Anything other than HTTP 200 indicates a request error.
         * In this case, we again ask the user to try again later.
         **/
        if ( $responseCode !== 200 ) {

            $return = [
                'success' => false,
                'message' => 'Failed to get content due to an error: HTTP ' . $responseCode . PHP_EOL . 'URL: ' . $url
            ];
            wp_send_json( $return );
            wp_die();

        }

        curl_close( $curl );

        return json_decode( $json, true );

    }

    /**
     * Combine multiple audio files to one .mp3.
     *
     * @param $files - Audio files for gluing into one big.
     * @param $post_id - ID of the Post/Page.
     *
     * @since 1.0.0
     * @access public
     **/
    public function glue_audio( $files, $post_id ) {

        /** Get path to upload folder. */
        $upload_dir     = wp_get_upload_dir();
        $upload_basedir = $upload_dir['basedir'];

        /** Path to post audio file. */
        $audio_file = $upload_basedir . '/speaker/post-' . $post_id . '.mp3';

        /** Just in case it exist. */
        wp_delete_file( $audio_file );
        foreach ( $files as $audio ) {

            /** Add new audio part to file. */
            file_put_contents( $audio_file, file_get_contents( $audio ), FILE_APPEND );

            /** Remove temporary audio files. */
            wp_delete_file( $audio );
        }

    }

	/** @noinspection PhpFullyQualifiedNameUsageInspection */
    /**
     * Convert HTML to temporary audio file.
     *
     * @param $html - Content to be voiced.
     * @param $post_id - ID of the Post/Page.
     *
     * @return string
     * @throws \Google\ApiCore\ApiException
     * @throws \Google\ApiCore\ValidationException
     * @since 1.0.0
     * @access public
     **/
    public function part_speak( $html, $post_id ) {

        /** Instantiates a client. */
        $client = new TextToSpeechClient();

	    /** Strip all html tags, except SSML tags.  */
        $html = strip_tags( $html, '<p><break><say-as><sub><emphasis><voice>');

        /** Remove the white spaces from the left and right sides.  */
	    $html = trim( $html );

	    /** Convert HTML entities to their corresponding characters: &quot; => " */
        $html = html_entity_decode( $html );

        /** Get language code and name from <voice> tag, or use default. */
	    list( $lang_code, $lang_name ) = XMLHelper::get_instance()->get_lang_params_from_tag( $html );

	    /** We don’t need <voice> tag anymore. */
	    $html = strip_tags( $html, '<p><break><say-as><sub><emphasis>');

        /** Force to SSML. */
	    $ssml = "<speak>";
	    $ssml .= $html;
	    $ssml .= "</speak>";

        /** Sets text to be synthesised. */
	    // $synthesisInputText = ( new SynthesisInput() )->setText( $html );
        $synthesisInputText = ( new SynthesisInput() )->setSsml( $ssml );

        /** Build the voice request, select the language. */
        $voice = ( new VoiceSelectionParams() )
            ->setLanguageCode( $lang_code )
            ->setName( $lang_name );

        /** Effects profile. */
        $effectsProfileId = $this->options['audio-profile'];

        /** Select the type of audio file you want returned. */
        $audioConfig = ( new AudioConfig() )
            ->setAudioEncoding( AudioEncoding::MP3 )
            ->setEffectsProfileId( [ $effectsProfileId ] )
            ->setSpeakingRate( $this->options['speaking-rate'] )
            ->setPitch( $this->options['pitch'] );

        /** Perform text-to-speech request on the text input with selected voice. */
        /** @noinspection PhpUnhandledExceptionInspection */
        $response = $client->synthesizeSpeech( $synthesisInputText, $voice, $audioConfig );

        /** The response's audioContent is binary. */
        $audioContent = $response->getAudioContent();

        /** Get path to upload folder. */
        $upload_dir     = wp_get_upload_dir();
        $upload_basedir = $upload_dir['basedir'];

        /** Path to audio file. */
        $audio_file = $upload_basedir . '/speaker/tmp-' . uniqid() . '-post-' . $post_id . '.mp3';
        file_put_contents( $audio_file, $audioContent );

        return $audio_file;
    }

	/**
	 * Divide parts by voice. One part voiced by one voice.
	 *
	 * @param array $parts HTML parts to be voiced.
	 *
	 * @return array() HTML parts to be voiced.
	 * @since 2.0.0
	 * @access public
	 */
	public function voice_divider( $parts ) {

		/** Array with parts splitted by voice. */
		$result = [];
		foreach ( $parts as $part ) {

		    /** Mark location of the cut. */
			$part = str_replace("<voice", "{|mdp|}<voice", $part );
			$part = str_replace("</voice>", "</voice>{|mdp|}", $part );

			/** Cut by marks. */
			$arr = explode( "{|mdp|}", $part );

			/** Clean the array. */
			$arr = array_filter( $arr );

			/** Combine results. */
			$result = array_merge( $result, $arr );

		}

		/** Fix broken html of each part. */
		foreach ( $result as &$el ) {
			$el = XMLHelper::get_instance()->repair_html( $el );
		}

		return $result;
	}

    /**
     * Prepare HTML for Google TTS.
     *
     * @param $html - Post/Page content to split.
     * @param int $max
     *
     * @return array() HTML parts to speech.
     * @since 1.0.0
     * @access public
     **/
    public function great_divider( $html, $max = 4500 ) {

        $parts = [];

        /** If the entire html is less than the quota. */
        if ( strlen( $html ) <= $max ) {
            $parts[] = $html;

            /** If we need to split HTML into parts. */
        } else {

            /** Divide HTML by closing tags '</' */
            $html_array = preg_split( '/(<\/)/', $html );
            $html_array = array_filter( $html_array );

            /** Fix broken tags, add '</' to all except first element. */
            $count = 0;
            foreach ( $html_array as $i => $el ) {
                $count ++;
                if ( $count == 1 ) {
                    continue;
                } // Skip first element.

                $html_array[ $i ] = '</' . $el;
            }

            /** Fix broken html. */
            foreach ( $html_array as $i => $el ) {
                $html_array[ $i ] = XMLHelper::get_instance()->repair_html( $el );
            }

            /** Remove empty elements. */
            $html_array = array_filter( $html_array );

            /** Divide into parts. */
            $current   = "";
            foreach ( $html_array as $i => $el ) {
                $previous = $current;
                $current   .= $el;
                if ( strlen( $current ) >= $max ) {
                    $parts[]  = $previous;
                    $current   = $el;
                }
            }
            $parts[] = $current;

        }

        return $parts;
    }

	/**
     * Remove muted elements by class "speaker-mute" or attribute speaker-mute="".
     *
     * @param $post_content - Post/Page content.
     *
     * @return string
     * @since 2.0.0
     * @access public
     *
     */
    public function remove_muted_html( $post_content ) {

        /** Hide DOM parsing errors. */
        libxml_use_internal_errors( true );
        libxml_clear_errors();

        /** Load the possibly malformed HTML into a DOMDocument. */
        $dom          = new DOMDocument();
        $dom->recover = true;
        $dom->loadHTML( '<?xml encoding="UTF-8"><body id="repair">' . $post_content . '</body>' ); // input UTF-8.

	    $selector = new DOMXPath( $dom );

	    /** Remove all elements with speaker-mute="" attribute. */
	    foreach( $selector->query( '//*[@speaker-mute]') as $e ) {
		    $e->parentNode->removeChild( $e );
	    }

	    /** Remove all elements with class="speaker-mute". */
	    foreach( $selector->query( '//*[contains(attribute::class, "speaker-mute")]' ) as $e ) {
		    $e->parentNode->removeChild( $e );
	    }

	    /** HTML without muted tags. */
	    $body = $dom->documentElement->lastChild;

	    return trim( XMLHelper::get_instance()->get_inner_html( $body ) );
    }

    /**
     * Setting Up Authentication.
     *
     * @since 1.0.0
     * @access public
     **/
    public function set_authentication_credentials() {

        $key_path = get_attached_file( $this->options['api-key-file'] );

        /** Setting Up Authentication. */
        putenv( 'GOOGLE_APPLICATION_CREDENTIALS=' . $key_path );
    }

    /**
     * Allow JSON files in the media library.
     *
     * @param $mime_types - Current array of mime types.
     *
     * @return array - Updated array of mime types.
     * @since 1.0.0
     * @access public
     */
    public function allow_json_uploads( $mime_types ) {

        /** Adding .json extension. */
        $mime_types['json'] = 'text/plain';

        return $mime_types;
    }

    /**
     * Show admin warning, if we need API Key.
     *
     * @since 1.0.0
     * @access public
     **/
    public function key_notice() {

	    /** Get current screen. */
	    $screen = get_current_screen();

	    /** Speaker Settings Page. */
	    if ( $screen->base == "toplevel_page_mdp_speaker_settings" ) {

		    /** Render "Before you start" message. */
		    UI::get_instance()->render_snackbar(
			    esc_html__( 'This plugin uses the Google Cloud Text-to-Speech API Key File. Set up your Google Cloud Platform project before the start.', 'speaker' ),
			    'warning', // Type
			    -1, // Timeout
			    true, // Is Closable
			    array( [ 'caption' => 'Get Key File', 'link' => 'https://docs.merkulov.design/about-key-file-for-the-voicer-wordpress-plugin/' ] ) // Buttons
		    );

        } else { ?>

            <div class="settings-error notice notice-warning">
                <p><strong><?php esc_html_e( 'Speaker: Before you begin', 'speaker' ); ?></strong></p>
                <p><?php esc_html_e( 'This plugin uses the Cloud Text-to-Speech API. You need to set up your Google Cloud Platform project and authorization before creating audio from text. Visit', 'speaker' ); ?>
                    <a href="https://docs.merkulov.design/about-key-file-for-the-voicer-wordpress-plugin/"
                       target="_blank"><?php esc_html_e( 'Online Documentation', 'speaker' ); ?></a> <?php esc_html_e( 'for more details.', 'speaker' ); ?>
                </p>
            </div>

            <?php

	    }

    }

    /**
     * Add admin menu for plugin settings.
     *
     * @since 1.0.0
     * @access public
     **/
    public function add_admin_menu() {

        add_menu_page(
            esc_html__( 'Speaker Settings', 'speaker' ),
            esc_html__( 'Speaker', 'speaker' ),
            'manage_options',
            'mdp_speaker_settings',
            [ $this, 'options_page' ],
            self::$url . 'images/speaker-logo.svg',
            '58.1961'// Always change digits after "." for different plugins.
        );
    }

    /**
     * Plugin Settings Page.
     *
     * @since 1.0.0
     * @access public
     **/
    public function options_page() {

        if ( ! current_user_can( 'manage_options' ) ) {
            return;
        } ?>
        <!--suppress HtmlUnknownTarget -->
        <form action='options.php' method='post'>
            <div class="wrap">

                <?php
                $tab = 'voice';
                if ( isset ( $_GET['tab'] ) ) {
                    $tab = $_GET['tab'];
                }

                /** Render "Speaker settings saved!" message. */
                $this->render_nags();

                /** Render Tabs Headers. */
                ?><section class="mdp-aside"><?php
                    $this->render_tabs( $tab );
                ?></section><?php

                /** Render Tabs Body. */
                ?><section class="mdp-tab-content"><?php

                    /** General Tab. */
                    if ( $tab == 'voice' ) {
                        echo '<h3>' . esc_html__( 'Voice Settings', 'speaker' ) . '</h3>';
                        settings_fields( 'SpeakerOptionsGroup' );
                        do_settings_sections( 'SpeakerOptionsGroup' );

                    } /** Design Tab. */
                    elseif ( $tab == 'design' ) {
                        echo '<h3>' . esc_html__( 'Design Settings', 'speaker' ) . '</h3>';
                        settings_fields( 'DesignOptionsGroup' );
                        do_settings_sections( 'DesignOptionsGroup' );

                    } /** Assignments Tab. */
                    elseif ( $tab == 'assignments' ) {
                        echo '<h3>' . esc_html__( 'Assignments Settings', 'speaker' ) . '</h3>';
                        settings_fields( 'SpeakerAssignmentsOptionsGroup' );
                        do_settings_sections( 'SpeakerAssignmentsOptionsGroup' );
                        AssignmentsTab::get_instance()->render_assignments();

                    } /** Activation Tab. */
                    elseif ( $tab == 'activation' ) {
                        settings_fields( 'SpeakerActivationOptionsGroup' );
                        do_settings_sections( 'SpeakerActivationOptionsGroup' );
                        PluginActivation::get_instance()->render_pid();

                    } /** Status tab. */
                    elseif ( $tab == 'status' ) {
                        echo '<h3>' . esc_html__( 'System Requirements', 'speaker' ) . '</h3>';
                        StatusTab::get_instance()->render_form();

                    } /** Uninstall Tab. */
                    elseif ( $tab == 'uninstall' ) {
                        echo '<h3>' . esc_html__( 'Uninstall Settings', 'speaker' ) . '</h3>';
                        UninstallTab::get_instance()->render_form();
                    }

                    ?>
                </section>
            </div>
        </form>

        <?php
    }

    /**
     * Render Tabs Headers.
     *
     * @param string $current - Selected tab key.
     *
     * @since 1.0.0
     * @access public
     */
    public function render_tabs( $current = 'voice' ) {

        /** Tabs array. */
        $tabs          = [];
        $tabs['voice'] = [
                'icon' => 'tune',
                'name' => esc_html__( 'Voice', 'speaker' )
        ];

        /** Show this tabs only if we have key file. */
        if ( $this->options['api-key-file'] ) {
            $tabs['design'] = [
                'icon' => 'brush',
                'name' => esc_html__( 'Design', 'speaker' )
            ];

            $tabs['assignments'] = [
                'icon' => 'flag',
                'name' => esc_html__( 'Assignments', 'speaker' )
            ];
        }

	    /** Activation tab enable only if plugin have Envato ID. */
	    $plugin_id = EnvatoItem::get_instance()->get_id();
	    if ( (int)$plugin_id > 0 ) {
		    $tabs['activation'] = [
			    'icon' => 'vpn_key',
			    'name' => esc_html__( 'Activation', 'speaker' )
		    ];
	    }

        $tabs['status'] = [
            'icon' => 'info',
            'name' => esc_html__( 'Status', 'speaker' )
        ];

        $tabs['uninstall'] = [
            'icon' => 'delete_sweep',
            'name' => esc_html__( 'Uninstall', 'speaker' )
        ];

        /** Render Tabs. */
        ?>
        <aside class="mdc-drawer">
            <div class="mdc-drawer__content">
                <nav class="mdc-list">

                    <div class="mdc-drawer__header mdc-plugin-fixed">
                        <!--suppress HtmlUnknownAnchorTarget -->
                        <a class="mdc-list-item mdp-plugin-title" href="#wpwrap">
                            <i class="mdc-list-item__graphic" aria-hidden="true">
                                <img src="<?php echo esc_attr( self::$url . 'images/logo-color.svg' ); ?>" alt="<?php echo esc_html__( 'Speaker', 'speaker' ) ?>">
                            </i>
                            <span class="mdc-list-item__text">
                                <?php echo esc_html__( 'Speaker', 'speaker' ) ?>
                                <sup><?php echo esc_html__( 'ver.', 'speaker' ) . esc_html( self::$version ); ?></sup>
                            </span>
                        </a>
                        <button type="submit" name="submit" id="submit"
                                class="mdc-button mdc-button--dense mdc-button--raised">
                            <span class="mdc-button__label"><?php echo esc_html__( 'Save changes', 'speaker' ) ?></span>
                        </button>
                    </div>

                    <hr class="mdc-plugin-menu">
                    <hr class="mdc-list-divider">
                    <h6 class="mdc-list-group__subheader"><?php echo esc_html__( 'Plugin settings', 'speaker' ) ?></h6>

                    <?php

                    // Plugin settings tabs
                    foreach ( $tabs as $tab => $value ) {
                        $class = ( $tab == $current ) ? ' mdc-list-item--activated' : '';
                        echo "<a class='mdc-list-item " . $class . "' href='?page=mdp_speaker_settings&tab=" . $tab . "'><i class='material-icons mdc-list-item__graphic' aria-hidden='true'>" . $value['icon'] . "</i><span class='mdc-list-item__text'>" . $value['name'] . "</span></a>";
                    }

                    /** Helpful links. */
                    $this->support_link();

                    /** Activation Status. */
                    PluginActivation::get_instance()->display_status();

                    ?>

                </nav>
            </div>
        </aside>
        <?php
    }

    /**
     * Generate Settings Page.
     *
     * @since 1.0.0
     * @access public
     **/
    public function settings_init() {

        /** Voice Tab. */
        register_setting( 'SpeakerOptionsGroup', 'mdp_speaker_settings' );
        add_settings_section( 'mdp_speaker_pluginPage_section', '', null, 'SpeakerOptionsGroup' );

        if ( $this->options['api-key-file'] ) {
            /** Current Language. */
            add_settings_field( 'current_language', esc_html__( 'Now used:', 'speaker' ), [$this, 'render_current_language'], 'SpeakerOptionsGroup', 'mdp_speaker_pluginPage_section' );

            /** Select Language. */
            add_settings_field( 'language', esc_html__( 'Select Language:', 'speaker' ), [$this, 'render_language'], 'SpeakerOptionsGroup', 'mdp_speaker_pluginPage_section' );

	        /** Audio profile. */
	        add_settings_field( 'audio_profile', esc_html__( 'Audio Profile:', 'speaker' ), [$this, 'render_audio_profile'], 'SpeakerOptionsGroup', 'mdp_speaker_pluginPage_section' );

            /** Speaking rate/speed. */
            add_settings_field( 'speaking_rate', esc_html__( 'Speaking Speed:', 'speaker' ), [$this, 'render_speaking_rate'], 'SpeakerOptionsGroup', 'mdp_speaker_pluginPage_section' );

            /** Pitch. */
            add_settings_field( 'pitch', esc_html__( 'Pitch:', 'speaker' ), [$this, 'render_pitch'], 'SpeakerOptionsGroup', 'mdp_speaker_pluginPage_section' );

            /** Before Audio. */
            add_settings_field( 'before_audio', esc_html__( 'Before Audio:', 'speaker' ), [$this, 'render_before_audio'], 'SpeakerOptionsGroup', 'mdp_speaker_pluginPage_section' );

            /** After Audio. */
            add_settings_field( 'after_audio', esc_html__( 'After Audio:', 'speaker' ), [$this, 'render_after_audio'], 'SpeakerOptionsGroup', 'mdp_speaker_pluginPage_section' );

            /** Create Design Tab. */
            register_setting( 'DesignOptionsGroup', 'mdp_speaker_design_settings' );
            add_settings_section( 'mdp_speaker_settings_page_design_section', '', null, 'DesignOptionsGroup' );

            /** Player Position. */
            add_settings_field( 'position', esc_html__( 'Player Position:', 'speaker' ), [$this, 'render_position'], 'DesignOptionsGroup', 'mdp_speaker_settings_page_design_section' );

            /** Player Style. */
            add_settings_field( 'style', esc_html__( 'Player Style:', 'speaker' ), [$this, 'render_style'], 'DesignOptionsGroup', 'mdp_speaker_settings_page_design_section' );

            /** Player background color. */
            add_settings_field( 'bgcolor', esc_html__( 'Background Color:', 'speaker' ), [$this, 'render_bgcolor'], 'DesignOptionsGroup', 'mdp_speaker_settings_page_design_section' );

            /** Download link. */
            add_settings_field( 'link', esc_html__( 'Download Link:', 'speaker' ), [$this, 'render_link'], 'DesignOptionsGroup', 'mdp_speaker_settings_page_design_section' );

            /** Create Assignments Tab. */
            AssignmentsTab::get_instance()->add_settings();

        }

	    /** API Key. */
	    add_settings_field( 'api_key', esc_html__( 'API Key File:', 'speaker' ), [$this, 'render_api_key'], 'SpeakerOptionsGroup', 'mdp_speaker_pluginPage_section' );

        /** Create Activation Tab. */
        PluginActivation::get_instance()->add_settings();

	    /** Create Status Tab. */
	    StatusTab::get_instance()->add_settings();

        /** Create Uninstall Tab. */
        UninstallTab::get_instance()->add_settings();

    }

    /**
     * Render "Settings Saved" nags.
     *
     * @since    2.0.0
     **/
    public function render_nags() {

        if ( ! isset( $_GET['settings-updated'] ) ) { return; }

	    if ( strcmp( $_GET['settings-updated'], "true" ) == 0 ) {

		    /** Render "Settings Saved" message. */
		    UI::get_instance()->render_snackbar( esc_html__( 'Settings saved!', 'speaker' ) );

	    }

	    if ( ! isset( $_GET['tab'] ) ) { return; }

        if ( strcmp( $_GET['tab'], "activation" ) == 0 ) {

            if ( PluginActivation::get_instance()->is_activated() ) {

	            /** Render "Activation success" message. */
	            UI::get_instance()->render_snackbar( esc_html__( 'Plugin activated successfully.', 'speaker' ), 'success', 5500 );

            } else {

	            /** Render "Activation failed" message. */
	            UI::get_instance()->render_snackbar( esc_html__( 'Invalid purchase code.', 'speaker' ), 'error', 5500 );

            }

        }

    }

    /**
     * Render Download link field.
     *
     * @since 2.0.0
     * @access public
     **/
    public function render_link() {

        $options = [
            'none' => esc_html__( 'Do not show', 'speaker' ),
            'backend' => esc_html__( 'Backend Only', 'speaker' ),
            'frontend' => esc_html__( 'Frontend Only', 'speaker' ),
            'backend-and-frontend' => esc_html__( 'Backend and Frontend', 'speaker' )
        ];

        /** Render select. */
        UI::get_instance()->render_select(
            $options,
            $this->options['link'], // Selected option.
            esc_html__( 'Download link', 'speaker' ),
            esc_html__( 'Position of the Download audio link', 'speaker' ),
            ['name' => 'mdp_speaker_design_settings[link]']
        );
    }

    /**
     * Render Before Audio field.
     *
     * @since 2.0.0
     * @access public
     **/
    public function render_before_audio() {

        /** Render input. */
        UI::get_instance()->render_input(
            $this->options['before_audio'],
            esc_html__( 'Before Audio', 'speaker'),
            esc_html__( 'Add text before audio(intro).', 'speaker' ),
            ['name' => 'mdp_speaker_settings[before_audio]']
        );

    }

    /**
     * Render After Audio field.
     *
     * @since 2.0.0
     * @access public
     **/
    public function render_after_audio() {

        /** Render input. */
        UI::get_instance()->render_input(
            $this->options['after_audio'],
            esc_html__( 'After Audio', 'speaker'),
            esc_html__( 'Add a text after audio(outro).', 'speaker' ),
            ['name' => 'mdp_speaker_settings[after_audio]']
        );

    }

    /**
     * Render Player Position field.
     *
     * @since 1.0.0
     * @access public
     **/
    public function render_position() {

        $options = [
            "before-content" => esc_html__( 'Before Content', 'speaker' ),
            "after-content" => esc_html__( 'After Content', 'speaker' ),
            "top-fixed" => esc_html__( 'Top Fixed', 'speaker' ),
            "bottom-fixed" => esc_html__( 'Bottom Fixed', 'speaker' ),
            "before-title" => esc_html__( 'Before Title', 'speaker' ),
            "after-title" => esc_html__( 'After Title', 'speaker' ),
            "shortcode" => esc_html__( 'Shortcode [speaker]', 'speaker' )
        ];

        /** Render select. */
        UI::get_instance()->render_select(
            $options,
            $this->options['position'], // Selected option.
            esc_html__('Position', 'speaker' ),
            esc_html__( 'Select the Player position or use shortcode.', 'speaker' ),
            ['name' => 'mdp_speaker_design_settings[position]']
        );

    }

    /**
     * Render Style field.
     *
     * @since 1.0.0
     * @access public
     **/
    public function render_style() {

        $options = [
            'speaker-round' => esc_html__( 'Round player', 'speaker' ),
            'speaker-rounded' => esc_html__( 'Rounded player', 'speaker' ),
            'speaker-squared' => esc_html__( 'Squared player', 'speaker' ),
            'speaker-wp-default' => esc_html__( 'WordPress default player', 'speaker' ),
            'speaker-browser-default' => esc_html__( 'Browser default player', 'speaker' )
        ];

        /** Render select. */
        UI::get_instance()->render_select(
            $options,
            $this->options['style'], // Selected option.
            esc_html__( 'Player Style', 'speaker' ),
            esc_html__( 'Select one of the Player styles', 'speaker' ),
            ['name' => 'mdp_speaker_design_settings[style]']
        );
    }

    /**
     * Render Player Background Color field.
     *
     * @since 1.0.0
     * @access public
     **/
    public function render_bgcolor() {

        /** Render colorpicker. */
        UI::get_instance()->render_colorpicker(
            $this->options['bgcolor'],
            esc_html__( 'Background Color', 'speaker' ),
            esc_html__( 'Select the Player background-color', 'speaker' ),
            [
                'name' => 'mdp_speaker_design_settings[bgcolor]',
                'id' => 'mdp-speaker-bgcolor',
                'readonly' => 'readonly'
            ]
        );

    }

    /**
     * Render Pitch field.
     *
     * @since 1.0.0
     * @access public
     **/
    public function render_pitch() {

        /** Render slider. */
        UI::get_instance()->render_slider(
                $this->options['pitch'],
                -20,
                20,
                0.1,
                esc_html__( 'Pitch', 'speaker'),
                esc_html__( 'Current pitch:', 'speaker') . ' <strong>' . esc_html( $this->options['pitch'] ) . '</strong>',
                [
                    'name' => 'mdp_speaker_settings[pitch]',
                    'class' => 'mdc-slider-width',
                    'id' => 'mdp_speaker_settings_pitch'
                ],
                false
        );

    }

    /**
     * Render Speaking Rate/Speed field.
     *
     * @since 1.0.0
     * @access public
     **/
    public function render_speaking_rate() {

        /** Render slider. */
	    UI::get_instance()->render_slider(
		    $this->options['speaking-rate'],
		    0.25,
		    4.0,
		    0.25,
		    esc_html__( 'Speaking Rate/Speed', 'speaker'),
		    esc_html__( 'Speaking rate:', 'speaker') . ' <strong>' . esc_html( $this->options['speaking-rate'] ) . '</strong><br>',
		    [
			    'name' => 'mdp_speaker_settings[speaking-rate]',
			    'class' => 'mdc-slider-width',
                'id' => 'mdp_speaker_settings_rate'
		    ],
		    false
	    );

    }

    /**
     * Render Audio Profile field.
     *
     * @since 1.0.0
     * @access public
     **/
    public function render_audio_profile() {

        $options = [
            'wearable-class-device' => esc_html__( 'Smart watches and other wearables, like Apple Watch, Wear OS watch', 'speaker' ),
            'handset-class-device' => esc_html__( 'Smartphones, like Google Pixel, Samsung Galaxy, Apple iPhone', 'speaker' ),
            'headphone-class-device' => esc_html__( 'Earbuds or headphones for audio playback, like Sennheiser headphones', 'speaker' ),
            'small-bluetooth-speaker-class-device' => esc_html__( 'Small home speakers, like Google Home Mini', 'speaker' ),
            'medium-bluetooth-speaker-class-device' => esc_html__( 'Smart home speakers, like Google Home', 'speaker' ),
            'large-home-entertainment-class-device' => esc_html__( 'Home entertainment systems or smart TVs, like Google Home Max, LG TV', 'speaker' ),
            'large-automotive-class-device' => esc_html__( 'Car speakers', 'speaker' ),
            'telephony-class-application' => esc_html__( 'Interactive Voice Response (IVR) systems', 'speaker' ),
        ];

        /** Render select. */
        UI::get_instance()->render_select(
            $options,
            $this->options['audio-profile'], // Selected option.
	        esc_html__( 'Audio Profile', 'speaker' ),
	        esc_html__( 'Optimize the synthetic speech for playback on different types of hardware.', 'speaker' ),
            ['name' => 'mdp_speaker_settings[audio-profile]']
        );

    }

    /**
     * Return language name by code.
     *
     * @param $lang_code - Google language code.
     *
     * @return mixed
     * @since 1.0.0
     * @access public
     */
    public function get_lang_by_code( $lang_code ) {

        if ( is_object( $lang_code ) ) {
            $lang_code = $lang_code[0];
        }

        /** @noinspection SpellCheckingInspection */
        $languages = [
            'da-DK'  => 'Danish (Dansk)',
            'nl-NL'  => 'Dutch (Nederlands)',
            'en-AU'  => 'English (Australian)',
            'en-GB'  => 'English (UK)',
            'en-US'  => 'English (US)',
            'fr-CA'  => 'French Canada (Français)',
            'fr-FR'  => 'French France (Français)',
            'de-DE'  => 'German (Deutsch)',
            'it-IT'  => 'Italian (Italiano)',
            'ja-JP'  => 'Japanese (日本語)',
            'ko-KR'  => 'Korean (한국어)',
            'nb-NO'  => 'Norwegian (Norsk)',
            'pl-PL'  => 'Polish (Polski)',
            'pt-BR'  => 'Portuguese Brazil (Português)',
            'pt-PT'  => 'Portuguese Portugal (Portugal)',
            'ru-RU'  => 'Russian (Русский)',
            'sk-SK'  => 'Slovak (Slovenčina)',
            'es-ES'  => 'Spanish (Español)',
            'sv-SE'  => 'Swedish (Svenska)',
            'tr-TR'  => 'Turkish (Türkçe)',
            'uk-UA'  => 'Ukrainian (Українська)',
            'ar-XA'  => 'Arabic (العربية)',
            'cs-CZ'  => 'Czech (Čeština)',
            'el-GR'  => 'Greek (Ελληνικά)',
            'en-IN'  => 'Indian English',
            'fi-FI'  => 'Finnish (Suomi)',
            'vi-VN'  => 'Vietnamese (Tiếng Việt)',
            'id-ID'  => 'Indonesian (Bahasa Indonesia)',
            'fil-PH' => 'Philippines (Filipino)',
            'hi-IN'  => 'Hindi (हिन्दी)',
            'hu-HU'  => 'Hungarian (Magyar)',
            'cmn-CN' => 'Chinese (官话)',
            'cmn-TW' => 'Taiwanese Mandarin (中文(台灣))'
        ];

        return $languages[ $lang_code ];
    }

    /**
     * Return Voice Type.
     *
     * @param $lang_name - Google voice name.
     *
     * @return string
     * @since 1.0.0
     * @access public
     */
    public function get_voice_type( $lang_name ) {
        $pos = strpos( $lang_name, 'Wavenet' );

        if ( $pos === false ) {
            return '<img src="' . self::$url . 'images/standard.svg" alt="' . esc_attr( 'Standard' ) . '">' . esc_html( 'Standard' );
        } else {
            return '<img src="' . self::$url . 'images/wavenet.svg" alt="' . esc_attr( 'WaveNet' ) . '">' . esc_html( 'WaveNet' );
        }

    }

    /**
     * Render API Key field.
     *
     * @since 1.0.0
     * @access public
     **/
    public function render_api_key() {

        if ( $this->options['api-key-file'] ) {
            $key_path = get_attached_file( $this->options['api-key-file'] );
            $key_name = basename( $key_path );
        }

        ?>
        <div class="mdp-upload">
            <div>
                <input type="hidden" name="mdp_speaker_settings[api-key-file]"
                       id="mdp_speaker_settings[api-key-file]"
                       value="<?php echo esc_attr( $this->options['api-key-file'] ); ?>"/>
                <button class="mdp-select-key-file-btn mdc-button mdc-button--outlined"><?php esc_html_e( 'Select Key File', 'speaker' ); ?></button>
                <div class="mdp-key-file-name">
                    <?php if ( isset( $key_name ) ) : ?>
                        <?php echo esc_html( $key_name ); ?>
                    <?php endif; ?>
                </div>
            </div>
        </div>

        <?php
    }

    /**
     * Render Current Language
     */
    public function render_current_language() {

        ?>
        <div class="mdp-now-used">
            <div>
                <strong><?php echo esc_attr( $this->options['language'] ); ?></strong>
            </div>
            <div>
                <audio controls="">
                    <source src="https://cloud.google.com/text-to-speech/docs/audio/<?php echo esc_attr( $this->options['language'] ); ?>.mp3" type="audio/mp3">
		            <?php esc_html_e( 'Your browser does not support the audio element.', 'speaker' ); ?>
                </audio>
            </div>
        </div>
        <?php

    }

	/** @noinspection PhpFullyQualifiedNameUsageInspection */
	/**
     * Render Language field.
     *
     * @return void
     *
     * @throws \Google\ApiCore\ApiException
     * @throws \Google\ApiCore\ValidationException
     * @since 1.0.0
     * @access public
     *
     * @noinspection PhpUnhandledExceptionInspection
     */
    public function render_language() {

        /** Create client object. */
        $client = new TextToSpeechClient();

        /** Perform list voices request. */
        $response = $client->listVoices();
        $voices   = $response->getVoices();

        if ( count( $voices ) == 0 ) {
            ?>
            <div class="mdp-key-error"><?php esc_html_e( 'The request failed. Looks like a problem with your API Key File ', 'speaker' ); ?></div><?php
            return;
        }

        ?>

        <div id="mdp-speaker-settings-language-select"></div>

        <div class="mdc-text-field-helper-line mdp-speaker-helper-padding">
            <div class="mdc-text-field-helper-text mdc-text-field-helper-text--persistent"><?php esc_html_e( 'The list includes both standard and', 'speaker' ); ?>
                <a href="https://cloud.google.com/text-to-speech/docs/wavenet"
                         target="_blank"><?php esc_html_e( 'WaveNet voices', 'speaker' ); ?></a>.
                <?php esc_html_e( 'WaveNet voices are higher quality voices with different', 'speaker' ); ?>
                <a href="https://cloud.google.com/text-to-speech/pricing"
                   target="_blank"><?php esc_html_e( 'pricing', 'speaker' ); ?></a>;
                <?php esc_html_e( 'in the list, they have the voice type "WaveNet".', 'speaker' ); ?>
            </div>
        </div>

        <table id="mdp-speaker-settings-language-tbl" class="display stripe hidden">
            <thead>
                <tr>
                    <th><?php esc_html_e( 'Language', 'speaker' ); ?></th>
                    <th><?php esc_html_e( 'Voice', 'speaker' ); ?></th>
                    <th><?php esc_html_e( 'Gender', 'speaker' ); ?></th>
                </tr>
            </thead>
            <tbody>
            <?php

            foreach ( $voices as $voice ) : ?>
                <tr <?php if ( $voice->getName() == $this->options['language'] ) { echo 'class="selected"'; } ?>>
                    <td class="mdp-lang-name">
                        <?php echo esc_html( $this->get_lang_by_code( $voice->getLanguageCodes() ) ); // Language. ?>
                    </td>
                    <td>
                        <span class="mdp-lang-code" title="<?php echo esc_html( $voice->getLanguageCodes()[0] ); // Language Code ?>"><?php echo esc_html( $voice->getLanguageCodes()[0] ); // Language Code  ?></span>
                        -
                        <span><?php echo $this->get_voice_type( $voice->getName() ); //Voice Type ?></span>
                        -
                        <span class="mdp-voice-name" title="<?php echo esc_html( $voice->getName() ); // Voice name ?>"><?php echo esc_html( substr( $voice->getName(), -1 ) ); // Voice Variant ?></span>
                    </td>
                    <td>
                        <?php
                        /** SSML voice gender values from TextToSpeech\V1\SsmlVoiceGender. */
                        $ssmlVoiceGender = [ 'SSML_VOICE_GENDER_UNSPECIFIED', 'Male', 'Female', 'Neutral' ];

                        echo '<span title="' . esc_attr( $ssmlVoiceGender[ $voice->getSsmlGender() ], 'speaker' ) . '"><img src="' . self::$url . 'images/' . strtolower( $ssmlVoiceGender[ $voice->getSsmlGender() ] ) . '.svg" alt="' . esc_attr( $ssmlVoiceGender[ $voice->getSsmlGender() ], 'speaker' ) . '">' . esc_html( $ssmlVoiceGender[ $voice->getSsmlGender() ] ) . '</span>'; ?>
                    </td>
                </tr>
                <?php
            endforeach;

            $client->close();

            ?>
            </tbody>

        </table>

        <input id="mdp-speaker-settings-language" type='hidden' name='mdp_speaker_settings[language]'
               value='<?php echo esc_attr( $this->options['language'] ); ?>'>
        <input id="mdp-speaker-settings-language-code" type='hidden' name='mdp_speaker_settings[language-code]'
               value='<?php echo esc_attr( $this->options['language-code'] ); ?>'>
        <?php
    }

    /**
     * Get plugin settings with default values.
     *
     * @access public
     * @since 1.0.0
     * @return void
     **/
    public function get_options() {

        /** Voice tab settings. */
        $options = get_option( 'mdp_speaker_settings' );

        /** Design tab settings. */
        $design_settings = get_option( 'mdp_speaker_design_settings' );

        /** Default values. */
        $defaults = [
            /** Voice Tab. */
            'api-key-file'  => '', // API Key file.
            'language'      => 'en-US-Standard-D', // Language.
            'language-code' => 'en-US', // Language Code.
            'audio-profile' => 'handset-class-device', // Audio profile.
            'speaking-rate' => '1', // Speaking rate/speed.
            'pitch'         => '0', // Pitch.
            'before_audio'  => '', // Before Audio.
            'after_audio'   => '', // After audio.

            /** Design Tab. */
            'position'      => 'before-content', // Player Position.
            'style'         => 'speaker-round', // Default style.
            'bgcolor'       => 'rgba(2, 83, 238, 1)', // Tooltip background color.
            'link'          => 'none', // Download Link.
        ];

        /** Some checks for key file. Use only JSON file. */
	    if ( isset( $options['api-key-file'] ) ) {
		    $key_path = get_attached_file( $options['api-key-file'] );
		    $ext      = pathinfo( $key_path, PATHINFO_EXTENSION );
		    if ( 'json' != $ext ) {
			    $options['api-key-file'] = '';
		    }
        }

        /** Design Tab. */
        /** Player position. */
        if ( isset( $design_settings['position'] ) ) {
            $options['position'] = $design_settings['position'];
        }

        /** Player Style. */
        if ( isset( $design_settings['style'] ) ) {
            $options['style'] = $design_settings['style'];
        }

        /** Player Background Color. */
        if ( isset( $design_settings['bgcolor'] ) ) {
            if ( $design_settings['bgcolor'] == '' ) {
                $options['bgcolor'] = '#0253ee';
            } else {
                $options['bgcolor'] = $design_settings['bgcolor'];
            }
        }

        /**  Download Link. */
        if ( isset( $design_settings['link'] ) ) {
            $options['link'] = $design_settings['link'];
        }

        $results = wp_parse_args( $options, $defaults );

        $this->options = $results;
    }

    /**
     * Loads the Speaker translated strings.
     *
     * @access public
     * @since 1.0.0
     * @return void
     **/
    public function load_textdomain() {
        load_plugin_textdomain( 'speaker', false, dirname( plugin_basename( __FILE__ ) ) . '/languages/' );
    }

    /**
     * Add plugin styles.
     *
     * @since   1.0.0
     * @return void
     */
    public function enqueue_styles() {
        wp_enqueue_style( 'mdp-speaker', self::$url . 'css/speaker' . self::$suffix . '.css', [], self::$version );
    }

    /**
     * Add plugin scripts.
     *
     * @return void
     * @since   1.0.0
     **/
    public function enqueue_scripts() {
        wp_enqueue_script( 'jquery' );
    }

    /**
     * Remove media element styles and scripts.
     *
     * @return string
     * @since   1.0.0
     */
    public function remove_media_element() {
        return '';
    }

    /**
     * Add CSS for admin area.
     *
     * @return void
     * @since   1.0.0
     **/
    public function admin_styles() {

        /** Add styles only on setting page */
        $screen = get_current_screen();

        /** Speaker Settings Page. */
        if ( $screen->base === "toplevel_page_mdp_speaker_settings" ) {
            wp_enqueue_style( 'merkulov-ui', self::$url . 'css/merkulov-ui.min.css', array(), self::$version );
            wp_enqueue_style( 'dataTables', self::$url . 'css/jquery.dataTables' . self::$suffix . '.css', [], self::$version );
            wp_enqueue_style( 'mdp-speaker-admin', self::$url . 'css/admin' . self::$suffix . '.css', [], self::$version );

        /** Edit Post/Page. */
        } elseif ( in_array( $screen->post_type, [ 'post', 'page' ] ) && $screen->base != 'edit' ) {

            wp_enqueue_style( 'mdp-speaker-admin-post', self::$url . 'css/admin-post' . self::$suffix . '.css', [], self::$version );

        /** Plugin install page, for style "View version details" popup. */
        } elseif ( 'plugin-install' === $screen->base ) {

            /** Styles only for our plugin. */
	        if ( isset( $_GET['plugin'] )  AND $_GET['plugin'] === 'speaker' ) {
		        wp_enqueue_style( 'mdp-speaker-plugin-install', self::$url . 'css/plugin-install' . self::$suffix . '.css', [], self::$version );
	        }
        }

    }

    /**
     * Add JS for admin area.
     *
     * @return void
     * @since   1.0.0
     **/
    public function admin_scripts() {

        /** Add styles only on setting page */
        $screen = get_current_screen();

        /** Speaker Settings Page. */
        if ( $screen->base == "toplevel_page_mdp_speaker_settings" ) {
            wp_enqueue_script( 'merkulov-ui', self::$url . 'js/merkulov-ui' . self::$suffix . '.js', array(), self::$version, true );
            wp_enqueue_media(); // WordPress Image library.
            wp_enqueue_script( 'dataTables', self::$url . 'js/jquery.dataTables' . self::$suffix . '.js', [ 'jquery' ], self::$version, true );
            wp_enqueue_script( 'mdp-speaker-admin', self::$url . 'js/admin' . self::$suffix . '.js', ['jquery', 'dataTables'], self::$version, true );

            /** Remove "Thank you for creating with WordPress" and WP version from plugin settings page. */
            add_filter( 'admin_footer_text', '__return_empty_string', 11 );
            add_filter( 'update_footer', '__return_empty_string', 11 );

            /** Edit Post/Page. */
        } elseif ( in_array( $screen->post_type, [ 'post', 'page' ] ) && $screen->base != 'edit' ) {
            wp_enqueue_script( 'mdp-admin-post', self::$url . 'js/admin-post' . self::$suffix . '.js', [ 'jquery' ], self::$version, true );

            /** Set Nonce. */
            $ajax_nonce = wp_create_nonce( 'speaker-nonce' );

            /** Current post ID. */
            $id = get_the_ID();

            /** Get URL to upload folder. */
            $upload_dir     = wp_get_upload_dir();
            $upload_baseurl = $upload_dir['baseurl'];

            /** URL to post audio file. */
            $audio_url = $upload_baseurl . '/speaker/';

            wp_localize_script( 'mdp-admin-post', 'mdp_speaker', [
                'post_id'    => $id,
                'ajax_nonce' => $ajax_nonce,
                'audio_url'  => $audio_url
            ] );
        }

    }

    /**
     * Displays useful links for an activated and non-activated plugin.
     *
     * @since 1.0.3
     **/
    public function support_link() { ?>

        <hr class="mdc-list-divider">
        <h6 class="mdc-list-group__subheader"><?php echo esc_html__( 'Helpful links', 'speaker' ) ?></h6>

        <a class="mdc-list-item" href="https://docs.merkulov.design/tag/speaker/" target="_blank">
            <i class="material-icons mdc-list-item__graphic" aria-hidden="true"><?php echo esc_html__( 'collections_bookmark' ) ?></i>
            <span class="mdc-list-item__text"><?php echo esc_html__( 'Documentation', 'speaker' ) ?></span>
        </a>

        <?php if ( PluginActivation::get_instance()->is_activated() ) : /** Activated. */ ?>
            <a class="mdc-list-item" href="https://1.envato.market/speaker-support" target="_blank">
                <i class="material-icons mdc-list-item__graphic" aria-hidden="true"><?php echo esc_html__( 'mail' ) ?></i>
                <span class="mdc-list-item__text"><?php echo esc_html__( 'Get help', 'speaker' ) ?></span>
            </a>
            <a class="mdc-list-item" href="https://1.envato.market/cc-downloads" target="_blank">
                <i class="material-icons mdc-list-item__graphic" aria-hidden="true"><?php echo esc_html__( 'thumb_up' ) ?></i>
                <span class="mdc-list-item__text"><?php echo esc_html__( 'Rate this plugin', 'speaker' ) ?></span>
            </a>
        <?php endif; ?>

        <a class="mdc-list-item" href="https://1.envato.market/cc-merkulove" target="_blank">
            <i class="material-icons mdc-list-item__graphic" aria-hidden="true"><?php echo esc_html__( 'store' ) ?></i>
            <span class="mdc-list-item__text"><?php echo esc_html__( 'More plugins', 'speaker' ) ?></span>
        </a>
        <?php

    }

    /**
     * Run when the plugin is activated.
     *
     * @static
     * @since 2.0.0
     **/
    public static function on_activation() {

        /** Security checks. */
        if ( ! current_user_can( 'activate_plugins' ) ) {
            return;
        }

        $plugin = isset( $_REQUEST['plugin'] ) ? $_REQUEST['plugin'] : '';
        check_admin_referer( "activate-plugin_{$plugin}" );

	    /** Send install Action to our host. */
	    Helper::get_instance()->send_action( 'install', 'speaker', self::$version );

    }

    /**
     * Main Speaker Instance.
     *
     * Insures that only one instance of Speaker exists in memory at any one time.
     *
     * @static
     * @return Speaker
     * @since 1.0.0
     **/
    public static function get_instance() {
        if ( ! isset( self::$instance ) && ! ( self::$instance instanceof Speaker ) ) {
            self::$instance = new Speaker;
        }

        return self::$instance;
    }

    /**
     * Throw error on object clone.
     *
     * The whole idea of the singleton design pattern is that there is a single
     * object therefore, we don't want the object to be cloned.
     *
     * @return void
     * @since 1.0.0
     * @access protected
     **/
    public function __clone() {
        /** Cloning instances of the class is forbidden. */
        _doing_it_wrong( __FUNCTION__, esc_html__( 'The whole idea of the singleton design pattern is that there is a single object therefore, we don\'t want the object to be cloned.', 'speaker' ), self::$version );
    }

    /**
     * Disable unserializing of the class.
     *
     * The whole idea of the singleton design pattern is that there is a single
     * object therefore, we don't want the object to be unserialized.
     *
     * @return void
     * @since 1.0.0
     * @access protected
     **/
    public function __wakeup() {
        /** Unserializing instances of the class is forbidden. */
        _doing_it_wrong( __FUNCTION__, esc_html__( 'The whole idea of the singleton design pattern is that there is a single object therefore, we don\'t want the object to be unserialized.', 'speaker' ), self::$version );
    }

} // End Class Speaker.

/** Run when the plugin is activated. */
register_activation_hook( __FILE__, ['Merkulove\Speaker', 'on_activation'] );

/** Run Speaker class. */
Speaker::get_instance();
