<?php
/**
 * Create an audio version of your posts, with a selection of 194 voices across 35+ languages and variants.
 * Exclusively on Envato Market: https://1.envato.market/speaker
 *
 * @encoding        UTF-8
 * @version         2.0.5
 * @copyright       Copyright (C) 2018 - 2019 Merkulove ( https://merkulov.design/ ). All rights reserved.
 * @license         Envato License https://1.envato.market/KYbje
 * @contributors    Alexander Khmelnitskiy (info@alexander.khmelnitskiy.ua), Dmitry Merkulov (dmitry@merkulov.design)
 * @support         dmitry@merkulov.design
 **/

namespace Merkulove\Speaker;

/** Exit if accessed directly. */
if ( ! defined( 'ABSPATH' ) ) {
	header( 'Status: 403 Forbidden' );
	header( 'HTTP/1.1 403 Forbidden' );
	exit;
}

use Merkulove\Speaker as Speaker;

/**
 * SINGLETON: Class used to implement shortcodes.
 *
 * @since 2.0.0
 * @author Alexandr Khmelnytsky (info@alexander.khmelnitskiy.ua)
 **/
final class Shortcodes {

	/**
	 * The one true Shortcodes.
	 *
	 * @var Shortcodes
	 * @since 1.0.0
	 **/
	private static $instance;

	/**
	 * Sets up a new Shortcodes instance.
	 *
	 * @since 1.0.0
	 * @access public
	 **/
	private function __construct() {

		/** Initializes shortcodes. */
		add_action( 'init', [$this, 'shortcodes_init'] );

	}

	/**
	 * Initializes shortcodes.
	 *
	 * @since 2.0.0
	 * @access public
	 * @return void
	 **/
	public function shortcodes_init() {

		/** Add player by shortcode [speaker] or [speaker id=POST_ID] */
		add_shortcode( 'speaker', [ $this, 'speaker_shortcode' ] );

		/** Speaker Mute Shortcode. [speaker-mute]...[/speaker-mute] */
		add_shortcode( 'speaker-mute', [ $this, 'speaker_mute_shortcode' ] );

		/** Speaker Break Shortcode. [speaker-break time="2s"] */
		add_shortcode( 'speaker-break', [ $this, 'speaker_break_shortcode' ] );

		/** Speaker say‑as Shortcode. [speaker-say‑as interpret-as="cardinal"][/speaker-say‑as] */
		add_shortcode( 'speaker-say‑as', [ $this, 'speaker_say_as_shortcode' ] );

		/** Speaker sub Shortcode. [speaker-sub alias="World Wide Web Consortium"]W3C[/speaker-sub] */
		add_shortcode( 'speaker-sub', [ $this, 'speaker_sub_shortcode' ] );

		/** Speaker emphasis Shortcode. [speaker-emphasis level="moderate"]This is an important announcement.[/speaker-emphasis] */
		add_shortcode( 'speaker-emphasis', [ $this, 'speaker_emphasis_shortcode' ] );

		/** Speaker voice Shortcode. [speaker-voice name="en-GB-Wavenet-C"]I am not a real human.[/speaker-voice] */
		add_shortcode( 'speaker-voice', [ $this, 'speaker_voice_shortcode' ] );

	}

	/**
	 * Add Speaker voice by shortcode [speaker-voice name="en-GB-Wavenet-B"]My new voice.[/speaker-voice].
	 *
	 * @param $atts - An associative array of attributes specified in the shortcode.
	 *
	 * @param $content
	 *
	 * @return string
	 * @since 2.0.0
	 * @access public
	 */
	public function speaker_voice_shortcode( $atts, $content ) {

		/** White list of options with default values. */
		$atts = shortcode_atts( [
			'name' => ''
		], $atts );

		/** If we don't have name, exit. */
		if ( ! $atts['name'] ) {
			return do_shortcode( $content );
		}

		/** Extra protection from the fools */
		$atts['name'] = trim( strip_tags( $atts['name'] ) );

		/** Show shortcodes only for our parser. Hide on frontend. */
		if ( isset( $_GET['speaker_ssml'] )  AND $_GET['speaker_ssml'] ) {
			return '<voice name="' . esc_attr( $atts['name'] ) . '">' . do_shortcode( $content ) . '</voice>';
		} else {
			return do_shortcode( $content );
		}

	}

	/**
	 * Add Speaker emphasis by shortcode [speaker-emphasis level="reduced"]Some information[/speaker-emphasis].
	 *
	 * @param $atts - An associative array of attributes specified in the shortcode.
	 *
	 * @param $content - Shortcode content when using the closing shortcode construct: [foo] shortcode text [/ foo].
	 *
	 * @return string
	 * @since 2.0.0
	 * @access public
	 */
	public function speaker_emphasis_shortcode( $atts, $content ) {

		/** White list of options with default values. */
		$atts = shortcode_atts( [
			'level' => 'none'
		], $atts );

		/** Extra protection from the fools */
		$atts['level'] = trim( strip_tags( $atts['level'] ) );

		/** Show shortcodes only for our parser. Hide on frontend. */
		if ( isset( $_GET['speaker_ssml'] )  AND $_GET['speaker_ssml'] ) {
			return '<emphasis level="' . esc_attr( $atts['level'] ) . '">' . do_shortcode( $content ) . '</emphasis>';
		} else {
			return do_shortcode( $content );
		}

	}

	/**
	 * Add Speaker sub by shortcode [speaker-sub alias="Second World War"]WW2[/speaker-sub].
	 *
	 * @param $atts - An associative array of attributes specified in the shortcode.
	 *
	 * @param $content - Shortcode content when using the closing shortcode construct: [foo] shortcode text [/ foo].
	 *
	 * @return string
	 * @since 2.0.0
	 * @access public
	 */
	public function speaker_sub_shortcode( $atts, $content ) {

		/** White list of options with default values. */
		$atts = shortcode_atts( [
			'alias' => ''
		], $atts );

		/** If we don't have alias, exit. */
		if ( ! $atts['alias'] ) {
			return do_shortcode( $content );
		}

		/** Extra protection from the fools */
		$atts['alias'] = trim( strip_tags( $atts['alias'] ) );

		/** Show shortcodes only for our parser. Hide on frontend. */
		if ( isset( $_GET['speaker_ssml'] )  AND $_GET['speaker_ssml'] ) {
			return '<sub alias="' . esc_attr( $atts['alias'] ) . '">' . do_shortcode( $content ) . '</sub>';
		} else {
			return do_shortcode( $content );
		}

	}

	/**
	 * Add Speaker say‑as by shortcode [speaker-say‑as interpret-as="ordinal"][/speaker-say‑as].
	 *
	 * @param $atts - An associative array of attributes specified in the shortcode.
	 *
	 * @param $content
	 *
	 * @return string
	 * @since 2.0.0
	 * @access public
	 */
	public function speaker_say_as_shortcode( $atts, $content ) {

		/** White list of options with default values. */
		$atts = shortcode_atts( [
			'interpret-as' => '',
			'format' => '',
			'detail' => ''
		], $atts );

		/** If we don't have interpret-as, exit. */
		if ( ! $atts['interpret-as'] ) {
			return do_shortcode( $content );
		}

		/** Extra protection from the fools */
		$atts['interpret-as'] = trim( strip_tags( $atts['interpret-as'] ) );
		$atts['format'] = trim( strip_tags( $atts['format'] ) );
		$atts['detail'] = trim( strip_tags( $atts['detail'] ) );

		$res = '<say-as interpret-as="' . $atts['interpret-as'] . '" ';

		if ( $atts['format'] ) {
			$res .= 'format="' . $atts['format'] . '" ';
		}

		if ( $atts['detail'] ) {
			$res .= 'detail="' . $atts['detail'] . '" ';
		}

		$res .= '>' . do_shortcode( $content ) . '</say-as>';

		/** Show shortcodes only for our parser. Hide on frontend. */
		if ( isset( $_GET['speaker_ssml'] )  AND $_GET['speaker_ssml'] ) {
			return $res;
		} else {
			return do_shortcode( $content );
		}

	}

	/**
	 * Add Speaker break by shortcode [speaker-break time="300ms"].
	 *
	 * @param $atts - An associative array of attributes specified in the shortcode.
	 *
	 * @return string
	 * @since 2.0.0
	 * @access public
	 **/
	public function speaker_break_shortcode( $atts ) {

		/** White list of options with default values. */
		$atts = shortcode_atts( [
			'time' => '500ms',
			'strength' => 'medium'
		], $atts );

		/** Extra protection from the fools */
		$atts['time'] = trim( strip_tags( $atts['time'] ) );
		$atts['strength'] = trim( strip_tags( $atts['strength'] ) );

		/** Show shortcodes only for our parser. Hide on frontend. */
		if ( isset( $_GET['speaker_ssml'] )  AND $_GET['speaker_ssml'] ) {
			return '<break time="' . esc_attr( $atts['time'] ) . '" strength="' . esc_attr( $atts['strength'] ) . '" />';
		} else {
			return '';
		}

	}

	/**
	 * Add Speaker mute by shortcode [speaker-mute]...[/speaker-mute].
	 *
	 * @param $atts - An associative array of attributes specified in the shortcode.
	 * @param $content - Shortcode content when using the closing shortcode construct: [foo] shortcode text [/ foo].
	 *
	 * @return string
	 * @since 1.0.0
	 * @access public
	 **/
	public function speaker_mute_shortcode( $atts, $content ) {

		/** Show shortcodes only for our parser. Hide on frontend. */
		if ( isset( $_GET['speaker_ssml'] )  AND $_GET['speaker_ssml'] ) {
			return '<span speaker-mute="">' . do_shortcode( $content ) . '</span>';
		} else {
			return do_shortcode( $content );
		}

	}

	/**
	 * Add player by shortcode [speaker].
	 *
	 * @param $atts - An associative array of attributes specified in the shortcode.
	 *
	 * @return bool|false|string
	 * @since 2.0.0
	 * @access public
	 */
	public function speaker_shortcode( $atts ) {

		/**
		 * If selected other settings, but we found shortcode.
		 * Show short code, but don't read it.
		 **/
		if ( Speaker::get_instance()->options['position'] != 'shortcode' ) {
			return '<span class="speaker-mute">[speaker]</span>';
		}

		$params = shortcode_atts( [ 'id' => '0' ], $atts );

		$id = (int) $params['id'];

		return Speaker::get_instance()->get_player( $id );
	}

	/**
	 * Main Shortcodes Instance.
	 *
	 * Insures that only one instance of Shortcodes exists in memory at any one time.
	 *
	 * @static
	 * @return Shortcodes
	 * @since 2.0.0
	 **/
	public static function get_instance() {
		if ( ! isset( self::$instance ) && ! ( self::$instance instanceof Shortcodes ) ) {
			self::$instance = new Shortcodes;
		}

		return self::$instance;
	}

	/**
	 * Throw error on object clone.
	 *
	 * The whole idea of the singleton design pattern is that there is a single
	 * object therefore, we don't want the object to be cloned.
	 *
	 * @return void
	 * @since 2.0.0
	 * @access protected
	 **/
	public function __clone() {
		/** Cloning instances of the class is forbidden. */
		_doing_it_wrong( __FUNCTION__, esc_html__( 'The whole idea of the singleton design pattern is that there is a single object therefore, we don\'t want the object to be cloned.', 'speaker' ), Speaker::$version );
	}

	/**
	 * Disable unserializing of the class.
	 *
	 * The whole idea of the singleton design pattern is that there is a single
	 * object therefore, we don't want the object to be unserialized.
	 *
	 * @return void
	 * @since 2.0.0
	 * @access protected
	 **/
	public function __wakeup() {
		/** Unserializing instances of the class is forbidden. */
		_doing_it_wrong( __FUNCTION__, esc_html__( 'The whole idea of the singleton design pattern is that there is a single object therefore, we don\'t want the object to be unserialized.', 'speaker' ), Speaker::$version );
	}

} // End Class Shortcodes.
